@@ -32,20 +32,22 @@
 #include "nghttp3_test_helper.h"
 
 static const MunitTest tests[] = {
-    munit_void_test(test_nghttp3_qpack_encoder_encode),
-    munit_void_test(test_nghttp3_qpack_encoder_encode_try_encode),
-    munit_void_test(test_nghttp3_qpack_encoder_still_blocked),
-    munit_void_test(test_nghttp3_qpack_encoder_set_dtable_cap),
-    munit_void_test(test_nghttp3_qpack_decoder_feedback),
-    munit_void_test(test_nghttp3_qpack_decoder_stream_overflow),
-    munit_void_test(test_nghttp3_qpack_huffman),
-    munit_void_test(test_nghttp3_qpack_huffman_decode_failure_state),
-    munit_void_test(test_nghttp3_qpack_decoder_reconstruct_ricnt),
-    munit_test_end(),
+  munit_void_test(test_nghttp3_qpack_encoder_encode),
+  munit_void_test(test_nghttp3_qpack_encoder_encode_try_encode),
+  munit_void_test(test_nghttp3_qpack_encoder_still_blocked),
+  munit_void_test(test_nghttp3_qpack_encoder_set_dtable_cap),
+  munit_void_test(test_nghttp3_qpack_decoder_feedback),
+  munit_void_test(test_nghttp3_qpack_decoder_stream_overflow),
+  munit_void_test(test_nghttp3_qpack_huffman),
+  munit_void_test(test_nghttp3_qpack_huffman_decode_failure_state),
+  munit_void_test(test_nghttp3_qpack_decoder_reconstruct_ricnt),
+  munit_void_test(test_nghttp3_qpack_decoder_read_encoder),
+  munit_void_test(test_nghttp3_qpack_encoder_read_decoder),
+  munit_test_end(),
 };
 
 const MunitSuite qpack_suite = {
-    "/qpack", tests, NULL, 1, MUNIT_SUITE_OPTION_NONE,
+  "/qpack", tests, NULL, 1, MUNIT_SUITE_OPTION_NONE,
 };
 
 static void check_decode_header(nghttp3_qpack_decoder *dec, nghttp3_buf *pbuf,
@@ -60,20 +62,20 @@ static void check_decode_header(nghttp3_qpack_decoder *dec, nghttp3_buf *pbuf,
   size_t i = 0;
 
   nread =
-      nghttp3_qpack_decoder_read_encoder(dec, ebuf->pos, nghttp3_buf_len(ebuf));
+    nghttp3_qpack_decoder_read_encoder(dec, ebuf->pos, nghttp3_buf_len(ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(ebuf), ==, nread);
 
   nghttp3_qpack_stream_context_init(&sctx, stream_id, mem);
 
   nread = nghttp3_qpack_decoder_read_request(
-      dec, &sctx, &qnv, &flags, pbuf->pos, nghttp3_buf_len(pbuf), 0);
+    dec, &sctx, &qnv, &flags, pbuf->pos, nghttp3_buf_len(pbuf), 0);
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(pbuf), ==, nread);
 
   for (; nghttp3_buf_len(rbuf);) {
     nread = nghttp3_qpack_decoder_read_request(
-        dec, &sctx, &qnv, &flags, rbuf->pos, nghttp3_buf_len(rbuf), 1);
+      dec, &sctx, &qnv, &flags, rbuf->pos, nghttp3_buf_len(rbuf), 1);
 
     assert_true(nread > 0);
 
@@ -117,13 +119,13 @@ static void decode_header_block(nghttp3_qpack_decoder *dec, nghttp3_buf *pbuf,
   nghttp3_qpack_stream_context_init(&sctx, stream_id, mem);
 
   nread = nghttp3_qpack_decoder_read_request(
-      dec, &sctx, &qnv, &flags, pbuf->pos, nghttp3_buf_len(pbuf), 0);
+    dec, &sctx, &qnv, &flags, pbuf->pos, nghttp3_buf_len(pbuf), 0);
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(pbuf), ==, nread);
 
   for (;;) {
     nread = nghttp3_qpack_decoder_read_request(
-        dec, &sctx, &qnv, &flags, rbuf->pos, nghttp3_buf_len(rbuf), 1);
+      dec, &sctx, &qnv, &flags, rbuf->pos, nghttp3_buf_len(rbuf), 1);
 
     assert_true(nread >= 0);
 
@@ -135,7 +137,6 @@ static void decode_header_block(nghttp3_qpack_decoder *dec, nghttp3_buf *pbuf,
       assert_ptrdiff(0, ==, nread);
       assert_false(flags & NGHTTP3_QPACK_DECODE_FLAG_EMIT);
       assert_size(0, ==, nghttp3_buf_len(rbuf));
-      assert_true(nghttp3_buf_len(&dec->dbuf) > 0);
 
       break;
     }
@@ -157,19 +158,19 @@ void test_nghttp3_qpack_encoder_encode(void) {
   nghttp3_qpack_encoder enc;
   nghttp3_qpack_decoder dec;
   nghttp3_nv nva[] = {
-      MAKE_NV(":path", "/rsrc.php/v3/yn/r/rIPZ9Qkrdd9.png"),
-      MAKE_NV(":authority", "static.xx.fbcdn.net"),
-      MAKE_NV(":scheme", "https"),
-      MAKE_NV(":method", "GET"),
-      MAKE_NV("accept-encoding", "gzip, deflate, br"),
-      MAKE_NV("accept-language", "en-US,en;q=0.9"),
-      MAKE_NV(
-          "user-agent",
-          "Mozilla/5.0 (Windows NT 10.0; Win64; x64)AppleWebKit/537.36(KHTML, "
-          "like Gecko) Chrome/63.0.3239.70 Safari/537.36"),
-      MAKE_NV("accept", "image/webp,image/apng,image/*,*/*;q=0.8"),
-      MAKE_NV("referer", "https://static.xx.fbcdn.net/rsrc.php/v3/yT/l/0,cross/"
-                         "dzXGESIlGQQ.css"),
+    MAKE_NV(":path", "/rsrc.php/v3/yn/r/rIPZ9Qkrdd9.png"),
+    MAKE_NV(":authority", "static.xx.fbcdn.net"),
+    MAKE_NV(":scheme", "https"),
+    MAKE_NV(":method", "GET"),
+    MAKE_NV("accept-encoding", "gzip, deflate, br"),
+    MAKE_NV("accept-language", "en-US,en;q=0.9"),
+    MAKE_NV(
+      "user-agent",
+      "Mozilla/5.0 (Windows NT 10.0; Win64; x64)AppleWebKit/537.36(KHTML, "
+      "like Gecko) Chrome/63.0.3239.70 Safari/537.36"),
+    MAKE_NV("accept", "image/webp,image/apng,image/*,*/*;q=0.8"),
+    MAKE_NV("referer", "https://static.xx.fbcdn.net/rsrc.php/v3/yT/l/0,cross/"
+                       "dzXGESIlGQQ.css"),
   };
   int rv;
   nghttp3_buf pbuf, rbuf, ebuf;
@@ -203,7 +204,7 @@ void test_nghttp3_qpack_encoder_encode(void) {
   assert_size(1, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
   ref =
-      *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 0);
+    *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 0);
 
   assert_uint64(5, ==, ref->max_cnt);
   assert_uint64(1, ==, ref->min_cnt);
@@ -250,13 +251,13 @@ void test_nghttp3_qpack_encoder_encode_try_encode(void) {
   const nghttp3_mem *mem = nghttp3_mem_default();
   nghttp3_qpack_encoder enc;
   nghttp3_nv nva[] = {
-      MAKE_NV(":path", "/foo"),
-      MAKE_NV(":authority", "example.com"),
-      MAKE_NV("authorization", "bearer token"),
-      MAKE_NV("priority", "i"),
-      MAKE_NV("cookie", "short00000000000000"),
-      MAKE_NV("cookie", "large000000000000000"),
-      MAKE_NV("nonstd", "non-standard-cookie"),
+    MAKE_NV(":path", "/foo"),
+    MAKE_NV(":authority", "example.com"),
+    MAKE_NV("authorization", "bearer token"),
+    MAKE_NV("priority", "i"),
+    MAKE_NV("cookie", "short00000000000000"),
+    MAKE_NV("cookie", "large000000000000000"),
+    MAKE_NV("nonstd", "non-standard-cookie"),
   };
   int rv;
   nghttp3_buf pbuf, rbuf, ebuf;
@@ -355,12 +356,12 @@ void test_nghttp3_qpack_encoder_still_blocked(void) {
   const nghttp3_mem *mem = nghttp3_mem_default();
   nghttp3_qpack_encoder enc;
   nghttp3_nv nva1[] = {
-      MAKE_NV(":status", "103"),
-      MAKE_NV("link", "foo"),
+    MAKE_NV(":status", "103"),
+    MAKE_NV("link", "foo"),
   };
   nghttp3_nv nva2[] = {
-      MAKE_NV(":status", "200"),
-      MAKE_NV("content-type", "text/foo"),
+    MAKE_NV(":status", "200"),
+    MAKE_NV("content-type", "text/foo"),
   };
   int rv;
   nghttp3_buf pbuf, rbuf, ebuf;
@@ -393,13 +394,13 @@ void test_nghttp3_qpack_encoder_still_blocked(void) {
   stream = nghttp3_qpack_encoder_find_stream(&enc, 0);
 
   ref =
-      *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 0);
+    *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 0);
 
   assert_true(nghttp3_ringbuf_len(&stream->refs) > 1);
   assert_uint64(ref->max_cnt, !=, nghttp3_qpack_stream_get_max_cnt(stream));
 
   ref =
-      *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 1);
+    *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 1);
 
   assert_uint64(ref->max_cnt, ==, nghttp3_qpack_stream_get_max_cnt(stream));
 
@@ -410,7 +411,7 @@ void test_nghttp3_qpack_encoder_still_blocked(void) {
   stream = nghttp3_qpack_encoder_find_stream(&enc, 0);
 
   ref =
-      *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 0);
+    *(nghttp3_qpack_header_block_ref **)nghttp3_ringbuf_get(&stream->refs, 0);
 
   assert_size(1, ==, nghttp3_ringbuf_len(&stream->refs));
   assert_uint64(ref->max_cnt, ==, nghttp3_qpack_stream_get_max_cnt(stream));
@@ -432,12 +433,12 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   nghttp3_qpack_decoder dec;
   nghttp3_buf pbuf, rbuf, ebuf;
   const nghttp3_nv nva1[] = {
-      MAKE_NV(":path", "/"),
-      MAKE_NV("date", "bar1"),
+    MAKE_NV(":path", "/"),
+    MAKE_NV("date", "bar1"),
   };
   const nghttp3_nv nva2[] = {
-      MAKE_NV(":path", "/"),
-      MAKE_NV("vary", "bar2"),
+    MAKE_NV(":path", "/"),
+    MAKE_NV("vary", "bar2"),
   };
   int rv;
   nghttp3_ssize nread;
@@ -466,8 +467,8 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   assert_size(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD,
               ==, enc.ctx.dtable_size);
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
   assert_uint64(1, ==, dec.ctx.next_absidx);
@@ -485,20 +486,18 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
 
   assert_int(0, ==, rv);
   assert_size(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD +
-                  strlen("vary") + strlen("bar2") +
-                  NGHTTP3_QPACK_ENTRY_OVERHEAD,
+                strlen("vary") + strlen("bar2") + NGHTTP3_QPACK_ENTRY_OVERHEAD,
               ==, enc.ctx.dtable_size);
   assert_uint64(2, ==, enc.ctx.next_absidx);
   assert_size(2, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
   assert_uint64(2, ==, dec.ctx.next_absidx);
   assert_size(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD +
-                  strlen("vary") + strlen("bar2") +
-                  NGHTTP3_QPACK_ENTRY_OVERHEAD,
+                strlen("vary") + strlen("bar2") + NGHTTP3_QPACK_ENTRY_OVERHEAD,
               ==, dec.ctx.dtable_size);
   assert_size(4096, ==, dec.ctx.max_dtable_capacity);
 
@@ -522,14 +521,13 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   assert_uint64(2, ==, enc.ctx.next_absidx);
   assert_size(2, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
   assert_uint64(2, ==, dec.ctx.next_absidx);
   assert_size(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD +
-                  strlen("vary") + strlen("bar2") +
-                  NGHTTP3_QPACK_ENTRY_OVERHEAD,
+                strlen("vary") + strlen("bar2") + NGHTTP3_QPACK_ENTRY_OVERHEAD,
               ==, dec.ctx.dtable_size);
   assert_size(4096, ==, dec.ctx.max_dtable_capacity);
 
@@ -550,16 +548,15 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   assert_size(0, ==, enc.ctx.max_dtable_capacity);
   assert_size(1, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
   /* decoder still has 2 entries because encoder does not emit Set
      Dynamic Table Capacity. */
   assert_uint64(2, ==, dec.ctx.next_absidx);
   assert_size(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD +
-                  strlen("vary") + strlen("bar2") +
-                  NGHTTP3_QPACK_ENTRY_OVERHEAD,
+                strlen("vary") + strlen("bar2") + NGHTTP3_QPACK_ENTRY_OVERHEAD,
               ==, dec.ctx.dtable_size);
   assert_size(4096, ==, dec.ctx.max_dtable_capacity);
 
@@ -581,8 +578,8 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   assert_size(SIZE_MAX, ==, enc.min_dtable_update);
   assert_size(0, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
   assert_uint64(2, ==, dec.ctx.next_absidx);
@@ -611,8 +608,8 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   assert_size(strlen("date") + strlen("bar1") + NGHTTP3_QPACK_ENTRY_OVERHEAD,
               ==, enc.ctx.dtable_size);
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
   assert_uint64(1, ==, dec.ctx.next_absidx);
@@ -644,8 +641,8 @@ void test_nghttp3_qpack_encoder_set_dtable_cap(void) {
   assert_size(1024, ==, enc.last_max_dtable_update);
   assert_size(1024, ==, enc.ctx.max_dtable_capacity);
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
   assert_uint64(2, ==, dec.ctx.next_absidx);
@@ -671,16 +668,16 @@ void test_nghttp3_qpack_decoder_feedback(void) {
   nghttp3_qpack_decoder dec;
   nghttp3_buf pbuf1, rbuf1, pbuf2, rbuf2, pbuf3, rbuf3, ebuf, dbuf;
   const nghttp3_nv nva1[] = {
-      MAKE_NV(":path", "/"),
-      MAKE_NV("date", "bar1"),
+    MAKE_NV(":path", "/"),
+    MAKE_NV("date", "bar1"),
   };
   const nghttp3_nv nva2[] = {
-      MAKE_NV(":path", "/"),
-      MAKE_NV("vary", "bar2"),
+    MAKE_NV(":path", "/"),
+    MAKE_NV("vary", "bar2"),
   };
   const nghttp3_nv nva3[] = {
-      MAKE_NV(":path", "/"),
-      MAKE_NV("link", "bar3"),
+    MAKE_NV(":path", "/"),
+    MAKE_NV("link", "bar3"),
   };
   int rv;
   nghttp3_ssize nread;
@@ -720,8 +717,8 @@ void test_nghttp3_qpack_decoder_feedback(void) {
   assert_int(0, ==, rv);
   assert_size(2, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
 
@@ -732,8 +729,8 @@ void test_nghttp3_qpack_decoder_feedback(void) {
 
   nghttp3_qpack_decoder_write_decoder(&dec, &dbuf);
 
-  nread = nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos,
-                                             nghttp3_buf_len(&dbuf));
+  nread =
+    nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos, nghttp3_buf_len(&dbuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&dbuf), ==, nread);
   /* This will unblock all streams because higher insert count is
@@ -749,8 +746,8 @@ void test_nghttp3_qpack_decoder_feedback(void) {
   nghttp3_buf_reset(&dbuf);
   nghttp3_qpack_decoder_write_decoder(&dec, &dbuf);
 
-  nread = nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos,
-                                             nghttp3_buf_len(&dbuf));
+  nread =
+    nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos, nghttp3_buf_len(&dbuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&dbuf), ==, nread);
   assert_size(0, ==, nghttp3_map_size(&enc.streams));
@@ -766,8 +763,8 @@ void test_nghttp3_qpack_decoder_feedback(void) {
   assert_int(0, ==, rv);
   assert_size(1, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
 
-  nread = nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos,
-                                             nghttp3_buf_len(&ebuf));
+  nread =
+    nghttp3_qpack_decoder_read_encoder(&dec, ebuf.pos, nghttp3_buf_len(&ebuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&ebuf), ==, nread);
 
@@ -777,8 +774,8 @@ void test_nghttp3_qpack_decoder_feedback(void) {
   assert_true(nghttp3_buf_len(&dbuf) > 0);
   assert_uint64(3, ==, dec.written_icnt);
 
-  nread = nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos,
-                                             nghttp3_buf_len(&dbuf));
+  nread =
+    nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos, nghttp3_buf_len(&dbuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&dbuf), ==, nread);
   assert_size(0, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
@@ -795,8 +792,8 @@ void test_nghttp3_qpack_decoder_feedback(void) {
 
   assert_true(nghttp3_buf_len(&dbuf) > 0);
 
-  nread = nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos,
-                                             nghttp3_buf_len(&dbuf));
+  nread =
+    nghttp3_qpack_encoder_read_decoder(&enc, dbuf.pos, nghttp3_buf_len(&dbuf));
 
   assert_ptrdiff((nghttp3_ssize)nghttp3_buf_len(&dbuf), ==, nread);
   assert_size(0, ==, nghttp3_qpack_encoder_get_num_blocked_streams(&enc));
@@ -851,7 +848,7 @@ void test_nghttp3_qpack_huffman(void) {
 
     nghttp3_qpack_huffman_decode_context_init(&ctx);
     nwrite =
-        nghttp3_qpack_huffman_decode(&ctx, dbuf, ebuf, (size_t)(end - ebuf), 1);
+      nghttp3_qpack_huffman_decode(&ctx, dbuf, ebuf, (size_t)(end - ebuf), 1);
     if (nwrite <= 0) {
       assert_true(nwrite > 0);
       continue;
@@ -898,3 +895,139 @@ void test_nghttp3_qpack_decoder_reconstruct_ricnt(void) {
 
   nghttp3_qpack_decoder_free(&dec);
 }
+
+void test_nghttp3_qpack_decoder_read_encoder(void) {
+  const nghttp3_mem *mem = nghttp3_mem_default();
+  nghttp3_qpack_decoder dec;
+  nghttp3_qpack_encoder enc;
+  int rv;
+  uint8_t b = 0x20; /* Set Dynamic Table Capacity */
+  size_t i;
+  nghttp3_ssize nread;
+  const nghttp3_nv nva[] = {
+    MAKE_NV("foo", "bar"),
+  };
+  nghttp3_buf pbuf, rbuf, ebuf;
+
+  /* Ensure limits */
+  rv = nghttp3_qpack_decoder_init(&dec, 4096, 1, mem);
+
+  assert_int(0, ==, rv);
+
+  for (i = 0; i < NGHTTP3_QPACK_MAX_ENCODERLEN; ++i) {
+    nread = nghttp3_qpack_decoder_read_encoder(&dec, &b, 1);
+
+    assert_ptrdiff(1, ==, nread);
+    assert_size(i + 1, ==, dec.uninterrupted_encoderlen);
+  }
+
+  nread = nghttp3_qpack_decoder_read_encoder(&dec, &b, 1);
+
+  assert_ptrdiff(NGHTTP3_ERR_QPACK_ENCODER_STREAM_ERROR, ==, nread);
+
+  nghttp3_qpack_decoder_free(&dec);
+
+  /* See variable cleared if one field section is decoded. */
+  nghttp3_buf_init(&pbuf);
+  nghttp3_buf_init(&rbuf);
+  nghttp3_buf_init(&ebuf);
+
+  rv = nghttp3_qpack_encoder_init(&enc, 0, mem);
+
+  assert_int(0, ==, rv);
+
+  rv = nghttp3_qpack_decoder_init(&dec, 4096, 1, mem);
+
+  assert_int(0, ==, rv);
+
+  for (i = 0; i < NGHTTP3_QPACK_MAX_ENCODERLEN; ++i) {
+    nread = nghttp3_qpack_decoder_read_encoder(&dec, &b, 1);
+
+    assert_ptrdiff(1, ==, nread);
+    assert_size(i + 1, ==, dec.uninterrupted_encoderlen);
+  }
+
+  rv = nghttp3_qpack_encoder_encode(&enc, &pbuf, &rbuf, &ebuf, 0, nva,
+                                    nghttp3_arraylen(nva));
+
+  assert_int(0, ==, rv);
+  assert_size(0, ==, nghttp3_buf_len(&ebuf));
+
+  decode_header_block(&dec, &pbuf, &rbuf, 0, mem);
+
+  assert_size(0, ==, dec.uninterrupted_encoderlen);
+
+  nread = nghttp3_qpack_decoder_read_encoder(&dec, &b, 1);
+
+  assert_ptrdiff(1, ==, nread);
+  assert_size(1, ==, dec.uninterrupted_encoderlen);
+
+  nghttp3_qpack_decoder_free(&dec);
+  nghttp3_qpack_encoder_free(&enc);
+  nghttp3_buf_free(&ebuf, mem);
+  nghttp3_buf_free(&rbuf, mem);
+  nghttp3_buf_free(&pbuf, mem);
+}
+
+void test_nghttp3_qpack_encoder_read_decoder(void) {
+  const nghttp3_mem *mem = nghttp3_mem_default();
+  nghttp3_qpack_encoder enc;
+  int rv;
+  uint8_t b = 0x40; /* Stream Cancellation */
+  size_t i;
+  nghttp3_ssize nread;
+  const nghttp3_nv nva[] = {
+    MAKE_NV("foo", "bar"),
+  };
+  nghttp3_buf pbuf, rbuf, ebuf;
+
+  /* Ensure limits */
+  rv = nghttp3_qpack_encoder_init(&enc, 4096, mem);
+
+  assert_int(0, ==, rv);
+
+  for (i = 0; i < NGHTTP3_QPACK_MAX_DECODERLEN; ++i) {
+    nread = nghttp3_qpack_encoder_read_decoder(&enc, &b, 1);
+
+    assert_ptrdiff(1, ==, nread);
+    assert_size(i + 1, ==, enc.uninterrupted_decoderlen);
+  }
+
+  nread = nghttp3_qpack_encoder_read_decoder(&enc, &b, 1);
+
+  assert_ptrdiff(NGHTTP3_ERR_QPACK_DECODER_STREAM_ERROR, ==, nread);
+
+  nghttp3_qpack_encoder_free(&enc);
+
+  /* See variable cleared if one field section is encoded. */
+  nghttp3_buf_init(&pbuf);
+  nghttp3_buf_init(&rbuf);
+  nghttp3_buf_init(&ebuf);
+
+  rv = nghttp3_qpack_encoder_init(&enc, 4096, mem);
+
+  assert_int(0, ==, rv);
+
+  for (i = 0; i < NGHTTP3_QPACK_MAX_DECODERLEN; ++i) {
+    nread = nghttp3_qpack_encoder_read_decoder(&enc, &b, 1);
+
+    assert_ptrdiff(1, ==, nread);
+    assert_size(i + 1, ==, enc.uninterrupted_decoderlen);
+  }
+
+  rv = nghttp3_qpack_encoder_encode(&enc, &pbuf, &rbuf, &ebuf, 0, nva,
+                                    nghttp3_arraylen(nva));
+
+  assert_int(0, ==, rv);
+  assert_size(0, ==, enc.uninterrupted_decoderlen);
+
+  nread = nghttp3_qpack_encoder_read_decoder(&enc, &b, 1);
+
+  assert_ptrdiff(1, ==, nread);
+  assert_size(1, ==, enc.uninterrupted_decoderlen);
+
+  nghttp3_qpack_encoder_free(&enc);
+  nghttp3_buf_free(&ebuf, mem);
+  nghttp3_buf_free(&rbuf, mem);
+  nghttp3_buf_free(&pbuf, mem);
+}
